\name{SDM}

\docType{class}

\alias{SDM-class}

\title{Class "SDM"}

\description{
Parent class for a number of models defined in the predicts package. This is a virtual Class, no objects may be direclty created from it. 
}



\keyword{classes}
